/* $Id$ */
/*
 *  UNIX-Connect, a ZCONNECT(r) Transport and Gateway/Relay.
 *  Copyright (C) 1993-94  Martin Husemann
 *  Copyright (C) 1995     Christopher Creutzig
 *  Copyright (C) 1994-98  Moritz Both
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ------------------------------------------------------------------------
 * Eine deutsche Zusammenfassung zum Copyright dieser Programme sowie der
 * GNU General Public License finden Sie in der Datei README.1st.
 * ------------------------------------------------------------------------
 *
 *  Bugreports, suggestions for improvement, patches, ports to other systems
 *  etc. are welcome. Contact the maintainer by e-mail:
 *  christopher@nescio.foebud.org or snail-mail:
 *  Christopher Creutzig, Im Samtfelde 19, 33098 Paderborn
 *
 *  There is a mailing-list for user-support:
 *   unix-connect@mailinglisten.im-netz.de,
 *  write a mail with subject "Help" to
 *   unix-connect-request@mailinglisten.im-netz.de
 *  for instructions on how to join this list.
 */


/*
 * zconv.c - Konvertiert einen ZCONNECT-Header nach RFC
 *
 * $Log$
 * Revision 1.24  2000/03/13 11:55:04  moritz
 * Behandlung von Point-User ersetzung ueberarbeitet
 *
 * Revision 1.23  2000/02/24 15:31:48  moritz
 * nderungen aus dem Projekt KeinZerberus2000 eingefgt
 *
 * Revision 1.22  1998/08/19 08:32:21  moritz
 * - bricht RFC-Headerzeilen nicht mehr mitten in quoted-strings um.
 *
 * Revision 1.21  1998/04/17 12:54:13  moritz
 * - Compilierungs-Zhler in gate/
 * - References: werden nicht mehr umgebrochen
 * - umbannant in UnixConnect II
 * - einige Bugfixes von der offiziellen Liste bernommen
 * - NOKOP wird korrekt behandelt
 * - base64-dekodierung von Headerzeilen, base64-dekodierung allg. verbessert
 *
 * Revision 1.20  1998/04/16 14:24:52  moritz
 * - Offizielle Version 0.76b1 hinzugefgt
 * - Erzeugt Message-ID, wenn keine da war bei Mail
 * - qp-De- und Enkodierung auch von X-Z- und X-ZC- Headerzeilen
 * - NOKOP wird korrekt behandelt: Keine Erzeugung von
 *   verrterischen To: oder Cc: Headerzeilen, wenn es
 *   vorhanden ist
 *
 * Revision 1.19  1998/03/04 02:04:01  moritz
 * - fix ROT/Path: handling
 *
 * Revision 1.18  1998/03/04 01:51:50  moritz
 * - code von mime.c in neues file mimew.c verschoben (nur fr Wandlung
 *         ZC->RFC)
 * - funktion 'count' in library verschoben
 * - Problem mit zuvielen/zuwenig Leerzeichen am Nachrichtenanfang behoben
 * - Q-Codierung von Headerzeilen weiter RFC 2047 angenhert (Zeilenlnge,
 *         Unterstriche)
 *
 * Revision 1.17  1998/02/16 17:26:31  moritz
 * - Betreff und ORG werden korrekt Umlaut-gewandelt
 *
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#ifdef HAS_STRING_H
# include <string.h>
#endif
#ifdef HAS_STRINGS_H
# include <strings.h>
#endif
#include "lib.h"
#ifdef NEED_VALUES_H
#include <values.h>
#endif
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <ctype.h>

#include "utility.h"
#include "header.h"
#include "hd_def.h"
#include "hd_nam.h"

#include "uulog.h"
#include "alias.h"
#include "version.h"
#include "boxstat.h"
#include "mime.h"
#include "mimew.h"
#include "zconv.h"
#include "cnumber.h"

char umlautstr[] = "\x94\x99\x84\x8e\x81\x9a\xe1",
 convertstr[] = "oeOeaeAeueUess";

char cancel[] = "cancel";

extern char eol[];
extern header_p pointuser;
extern char *pointsys;

/* get_pointuser():
 * liefert einen mit malloc allozierten String, der die korrekte 
 * Absender-Adresse eines Points darstellt. Die Adresse entspricht
 * der uebergebenen Adresse, wenn diese in der Adressenliste des
 * Points auftauscht.
 */

char *get_pointuser(const char *suggestion)
{
	header_p p;
	char *result = NULL;

	for (p = pointuser; p; p=p->other)
		if (adrmatch(suggestion, p->text))
			break;

	if (p == NULL)
	{
		/*
		 *  Point mit falscher Absenderangabe: ersetzen durch
		 *  den ersten Point-User: aus der Liste (das ist der Default-
		 *  User)
		 */
		if (pointuser != NULL)
			result = dstrdup(pointuser->text);
	}
	else 
	{
		/* Wenn er gefunden wurde, suggestion kopieren,
 		 * aber Adressteil durch unsere Schreibweise 
		 * ersetzen.
		 */
		result = dstrdup(suggestion);
		memcpy(result, p->text, strlen(p->text));
	}
	return result;
}

/*
 *  Liefert 1, wenn die beiden Adressteile (ohne Realname) identisch
 *  sind
 */
int adrmatch(const char *abs1, const char *abs2)
{
	while (toupper(*abs1) == toupper(*abs2)) {
		abs1++; abs2++;
		if (!*abs1 || !*abs2 || *abs1 == ' ' || *abs2 == ' ')
			break;
	}
	if (!*abs1 || *abs1 == ' ')
		return (!*abs2 || *abs2 == ' ');
	return 0;
}

/* Druckt EINE newsgroup nach EINEM Brett.
 * Gibt !=0 zurueck im Ungluecksfall.
 */
int printnewsgroup(char *brett, FILE *f) {
	char *s;
	static char buffer[MAXLINE];

	if (strchr(brett, '@')) 
		return 1;
	s = z_alias(brett);
	if (s) {
		fputs(s, f);
		return 0;
	}
	s = z_prefix(brett);
	if (s) {
		strcpy(buffer, s);
		strcat(buffer, (brett)+prefix_len);
	} else {
		strcpy(buffer, (brett)+1);
	}
	strlwr(buffer);
	for(s=buffer; *s; s++)
		if (*s == '/')
			*s = '.';
	fputs(buffer, f);
	return 0;
}

/* Druckt in file f alle newsgroups, die als Bretter in 
 * den Headerzeilen p ... p->other usw. stehen.
 */
static void printnewsgroups(header_p p, char *uuheader, FILE *f)
{
	header_p t;

	if (uuheader)
		fputs(uuheader, f);
	fputs(": ", f);
	for (t=p; t; t=t->other) {
		if (printnewsgroup(t->text, f) == 0
			&& t->other)
			putc(',', f);
	}
	fputs(eol, f);
}

header_p convheader(header_p hd, FILE *f)
{
	header_p p, t, to, cc, bcc;
	char *s, *test, *abs, *oabs, *sender;
	char *mime_name, *absname;
	int pos, nokop;
	extern int main_is_mail;
	int mail, news;

	minireadstat();
	nokop = 0;
	mail=0;
	news=0;
	to = find(HD_UU_U_TO, hd);
	cc = find(HD_UU_U_CC, hd);
	bcc = find(HD_UU_U_BCC, hd);
	p = find(HD_UU_U_RECEIVED, hd);
	if (p) {
		while (p) {
			foldputs(f, p->header+2, p->text);
			p = p->other;
		}
		hd = del_header(HD_UU_U_RECEIVED, hd);
	}
	p = find(HD_STAT, hd);
	if(p) {
		for(t=p; t; t=t->other) {
			if(strcasecmp(t->text,"NOKOP")==0)
				nokop = 1;
		}
	}
	if (main_is_mail && nokop)
		fprintf(f, "X-ZC-STAT: NOKOP%s", eol);

	p = find(HD_EMP, hd);
	if (p) {
		for (mail=0, news=0, t=p; t; t=t->other) {
			if (strchr(p->text, '@'))
				mail++;
			else
				news++;
		}
		if (mail && !to) {
			if (nokop) {
				/* NOKOP */
				if (mail == 1)
					/* Nur 1 Empf., in To: Zeile */
					foldputaddrs(f, HN_UU_TO, p);
				else
					/* mehrere Empf. - anonymisieren */
					fprintf(f, HN_UU_TO": undisclosed recipients:;%s", eol);
			}
			else
				/* no NOKOP */
				foldputaddrs(f, HN_UU_TO, p);
		}
		if (news) {
			/*
			 * Newsgroups: in News als Hauptempfaenger
			 */
		   	printnewsgroups(p, HN_UU_NEWSGROUPS, f);
		}
		hd = del_header(HD_EMP, hd);
	}
	p = find(HD_OEM, hd);
	if (p) {
		int oemmail;
				
		for (oemmail=0, t=p; t && !oemmail; t=t->other)
			if (strchr(p->text, '@'))
				oemmail = 1;
		if (oemmail) {
			foldputaddrs(f, HN_UU_ORIG_TO, p);
			hd = del_header(HD_OEM, hd);
		}
	}
	p = find(HD_KOP, hd);
	if (p) {
		extern int main_is_mail;
		int kopmail, kopnews;
		
		for (kopmail=0, kopnews = 0, t=p; t && !kopmail; t=t->other)
			if (strchr(p->text, '@'))
				kopmail = 1;
			else
				kopnews = 1;
		/* drucke Cc:, wenn es nicht schon da war, es Mail KOP:
		 * gibt UND NOKOP nicht da war 
		 */
		if (kopmail && !cc && !nokop) {
			foldputaddrs(f, HN_UU_CC, p);
			hd = del_header(HD_KOP, hd);
		}
		else
		if (main_is_mail && kopnews) {
			/* Newsgroups: als Kopienempfaenger in der Mail */
			printnewsgroups(p, HN_UU_NEWSGROUPS, f);
			hd = del_header(HD_KOP, hd);
		}
		else {
			/* Wenn wir die Headerzeile nicht bearbeitet haben, 
			 * wird sie als X-ZC-KOP: ausgegeben. 
			 * Das wollen wir aber NICHT mehr:
			 */
			 hd=del_header(HD_KOP, hd);
		}
	}
	if (to) {
		foldputh(f, HN_UU_TO, to);
	}
	if (cc) {
		foldputh(f, HN_UU_CC, cc);
	}
	if (bcc) {
		foldputh(f, HN_UU_BCC, bcc);
	}
	p = find(HD_MID, hd);
	if (p) {
		fprintf(f, HN_UU_MESSAGE_ID": <%s>%s", p->text, eol);
		hd = del_header(HD_MID, hd);
	}
	p = find(HD_ERSETZT, hd);
	if (p) {
		fprintf(f, HN_UU_SUPERSEDES": <%s>%s", p->text, eol);
		hd = del_header(HD_ERSETZT, hd);
	}

	p = find(HD_CONTROL, hd);
	if (p) {
		/* CONTROL: nur auswerten, wenn es kein U-Control
		 * gibt und es nur einmal da ist
		 */
		t = find(HD_UU_U_CONTROL, hd);
		if (t || p->other)
			; 
		else {
			/* CONTROL ist nur einmal da */
			/* Parameter ansteuern. */
			for (s = p->text; *s && !isspace(*s); s++);
			for (           ; *s &&  isspace(*s); s++);

			if (strncasecmp(p->text,cancel,sizeof(cancel)-1) == 0) {
				fprintf(f, HN_UU_CONTROL": cancel <%s>%s", s, eol);
			}
			else if (strncasecmp(p->text, "ADD", sizeof("ADD")-1) == 0) {
				fputs(HN_UU_CONTROL": newgroup ", f);
				printnewsgroup(s, f);
				fputs(eol, f);
			}
			else if (strncasecmp(p->text, "DEL", sizeof("DEL")-1) == 0) {
				fputs(HN_UU_CONTROL": rmgroup ", f);
				printnewsgroup(s, f);
				fputs(eol, f);
			}
			else {
				fprintf(f, "X-ZC-"HN_CONTROL": %s%s", p->text, eol);
			}
		}
		hd = del_header(HD_CONTROL, hd);
	}
	abs = NULL;
	sender = NULL;
	p = find(HD_OAB, hd);
	if (p && main_is_mail) {
		/* Bei Mail wird OAB zum From:
		 * bei news ist das nicht erwuenscht.
		 * (bei WAB:/ABS: passt Sender:/From:,
		 * fuer ABS:/OAB: bigt es in RFC kein passendes
		 * Pendant, sodass wir den OAB: loeschen muessen.)
		 */
		abs = strdup(p->text);
		p = find(HD_ABS, hd);
		if (p) {
			sender = dstrdup(p->text);
		}
		hd = del_header(HD_OAB, hd);
		hd = del_header(HD_ABS, hd);
	} else {
		p = find(HD_ABS, hd);
		if (p) {
			abs = dstrdup(p->text);
			p = find(HD_WAB, hd);
			if(p)
			{
				sender = dstrdup(p->text);
			}
			hd = del_header(HD_ABS, hd);
			hd = del_header(HD_WAB, hd);
			hd = del_header(HD_OAB, hd);
		}
	}
	if (pointuser) {
		/* Bei Points ggf. Absender ersetzen */
		char *pu;
		if (sender) {
			pu = get_pointuser(sender);
			dfree(sender);
			sender = pu;
		}
		else {
			pu = get_pointuser(abs);
			dfree(abs);
			abs = pu;
		}
	}
	if(abs)
		oabs = dstrdup(abs);
	else /* fehlerhafte Eingabedatei, kein ABS:-Header */
	{
#ifdef LOG_ERRORS_IN_HEADERS
		fprintf(f, "X-Gate-Error: No ABS%s", eol);
#endif
		oabs = dstrdup("");
		abs = dstrdup("");
	}
	if (!abs || !oabs) {
		out_of_mem(__FILE__,__LINE__);
	}
	s = strchr(abs, ' ');
	if (s) *s = '\0';
	strlwr(abs);
	test = strrchr(abs, '@');
	if (test && strcmp(test, "@uucp")==0) {
		*test = '\0';
		test = strrchr(abs, '%');
		if (test) *test = '@';
		dfree(oabs);
		oabs = dstrdup(abs);
	} else
		if (s) 
			*s = ' ';

	pc2iso(oabs, strlen(oabs));
	mime_name=mime_address(oabs);
	foldputs(f, HN_UU_FROM, mime_name);
	dfree(mime_name);
	hd = del_header(HD_UU_U_FROM, hd);

	if (sender && !main_is_mail) {
		/* Bei news WAB: in Sender: umsetzen */
		pc2iso(sender, strlen(sender));
		mime_name=mime_address(sender);

		fprintf(f, HN_UU_SENDER": %s%s", mime_name, eol);
		dfree(mime_name);
	}

	/* ROT wird zu Path in News;
	 * ROT wird als X-ZC- durchgereicht in Mail
	 *   (allerdings erst, wenn die Rueckwandlung sichergestellt ist)
	 * Wenn "Points" mit uns telefonieren, ersetzen wir ROT.
	 */
	p = find(HD_ROT, hd);
	if (main_is_mail == 0) {
		/* News */
		if (pointuser) {
			/* Points bekommen immer pointname!not-for-mail
			 * in den Path
			 */
			fprintf(f, HN_UU_PATH": %s!not-for-mail%s", pointsys,
							eol);
		}
		else
		if (!p || !(p->text)) {
			/* Wir haben keien ROT oder er ist leer -
			 * dann erzeugen wir Path:
			 */
			absname = dstrdup(oabs);
			s = strchr(absname, '@');
			if (s) {
				*s++ = '\0';
				fprintf(f,HN_UU_PATH": %s!%s%s", s, absname, eol);
			} else {
				fprintf(f,HN_UU_PATH": %s%s", absname, eol);
			}
			dfree(absname);
		} 
		else {
			s = strchr(p->text, '!');
			if(s)
				fprintf(f, HN_UU_PATH": %s%s", p->text, eol);
			else
				fprintf(f, HN_UU_PATH": %s!not-for-mail%s", p->text, eol);
		}
		hd=del_header(HD_ROT, hd);
	} 
	/* FIXME: Die folgende Zeile muss raus, wenn die Rueckwandlung
	 * funktioniert
	 */
	hd = del_header(HD_ROT, hd);

	p = find(HD_EB, hd);
	if (p) {
		header_p t;
		for(t=p; t; t=t->other) {
			if (*(t->text)) 
			{
				mime_name=mime_address(t->text);
				fprintf(f, HN_UU_RETURN_RECEIPT_TO ": %s%s", 
						mime_name, eol);
				dfree(mime_name);
			} else {
				mime_name=mime_address(t->text);
				fprintf(f, HN_UU_RETURN_RECEIPT_TO ": %s%s", 
						mime_name, eol);
				dfree(mime_name);
			}
		}
		hd = del_header(HD_EB, hd);
	}
	dfree(abs);
	dfree(oabs);
	if (sender)
		dfree(sender);
	p = find(HD_ORG, hd);
	if (p) {
		pc2iso(p->text, strlen(p->text));
		mime_name=mime_encode(p->text, sizeof(HN_UU_ORGANIZATION));
		foldputs(f, HN_UU_ORGANIZATION, mime_name);
		dfree(mime_name);
		hd = del_header(HD_ORG, hd);
	}
	p = find(HD_BET, hd);
	if (p) {
		/* Umlaute werden nicht mit ae... ausgegeben, um
		 * ein transparentes Gating zu ermglichen. */
		char *encoded;
		pc2iso(p->text, strlen(p->text));
		encoded = mime_encode(p->text, sizeof(HN_UU_SUBJECT));
		foldputs(f, HN_UU_SUBJECT, encoded);
		dfree(encoded);
		hd = del_header(HD_BET, hd);
	}
	p = find(HD_UU_U_DATE, hd);
	if (p) {
		fprintf(f, HN_UU_DATE": %s%s", p->text, eol);
		hd = del_header(HD_UU_U_DATE, hd);
		hd = del_header(HD_EDA, hd);
	} else {
		p = find(HD_EDA, hd);
		if (p) {
			char * buffer;
			int tz_hour, tz_min;

			buffer = str2eda(p->text, &tz_hour, &tz_min);
			if (buffer ==NULL)
				out_of_mem(__FILE__,__LINE__);
			fprintf(f, HN_UU_DATE": %s%s", buffer, eol);
			hd = del_header(HD_EDA, hd);
			dfree(buffer);
		}
	}
	p = find(HD_UU_U_EXPIRES, hd);
	if (p) {
		fprintf(f, HN_UU_EXPIRES": %s%s", p->text, eol);
		hd = del_header(HD_UU_U_EXPIRES, hd);
		hd = del_header(HD_LDA, hd);
	} else {
		p = find(HD_LDA, hd);
		if (p) {
			char *buffer;
			int tz_hour, tz_min;

			buffer = str2eda(p->text, &tz_hour, &tz_min);
			if(buffer ==NULL)
				out_of_mem(__FILE__,__LINE__);
			fprintf(f, HN_UU_EXPIRES": %s%s", buffer, eol);
			hd = del_header(HD_LDA, hd);
			dfree(buffer);
		}
	}
	p = find(HD_ANTWORT_AN, hd);
	if (p) {
		fputs(HN_UU_REPLY_TO": ", f);
	        for (pos = 10, t=p; t; t=t->other)
	        {
/* zur Sicherheit nach Brett schauen //// xxx */
		   mime_name=mime_address(t->text);
		   if(pos > 65)
		   {
		   	fprintf(f, "%s\t", eol);
		   	pos = 8;
		   }
		   fprintf(f, "%s%s", mime_name, t->other? ", " : "");
		   pos += strlen(mime_name) + 2;
		   dfree(mime_name);
		}
		fputs(eol,f);
		hd = del_header(HD_ANTWORT_AN, hd);
	}
	p = find(HD_DISKUSSION_IN, hd);
	if (p) {
		if (!p->other && strchr(p->text, '@')) {
	/* immer poster? //// xxx */
			fprintf(f, HN_UU_FOLLOWUP_TO": poster%s", eol);
		} else {
			printnewsgroups(p, HN_UU_FOLLOWUP_TO, f);
		}
		hd = del_header(HD_DISKUSSION_IN, hd);
	}
	p = find(HD_MAL, hd);
	if (p) {
		foldputs(f, HN_UU_X_MAILER, p->text);
		hd = del_header(HD_MAL, hd);
	}
	p = find(HD_BEZ, hd);
	if (p) {
		/* Weder RFC1036 noch verschiedene News-Reader verstehen
		 * einen Umbruch der References: Zeile.
		 */
		fputs(HN_UU_REFERENCES":", f);
		for ( ; p; p=p->other)
			fprintf(f, " <%s>", p->text);
		fputs(eol, f);
		hd = del_header(HD_BEZ, hd);
	}
	if (!pointuser) {
	    p = find(HD_ZNETZ_CONV, hd);
	    if (p) {
		fprintf(f, "X-Gateway: ZCONNECT %s.%s [" 
				MAILER " " VERSION "#%u], NETCALL3.8 %s",
				boxstat.boxname, boxstat.boxdomain, 
				cnumber, p->text);
	    } else {
		fprintf(f, "X-Gateway: ZCONNECT %s.%s [" 
				MAILER " " VERSION "#%u]",
				boxstat.boxname, boxstat.boxdomain, cnumber);
	    }
	    p = find(HD_GATE, hd);
	    if (p) {
		fprintf(f,",%s",eol);
	    	foldputs(f, NULL, p->text);
	    } else {
		fputs(eol, f);
	    }
	}
	hd = del_header(HD_ZNETZ_CONV, hd);
	hd = del_header(HD_ZNETZ_TEXT, hd);
	hd = del_header(HD_MAL, hd);
	hd = del_header(HD_UU_U_TO, hd);
	hd = del_header(HD_UU_U_CC, hd);
	hd = del_header(HD_UU_U_BCC, hd);
	hd = del_header(HD_GATE, hd);
	hd = del_header(HD_PRIO, hd);

	/* Fido: F-To: nach X-Comment-To: */
	p = find(HD_F_TO, hd);
	if (p) {
		foldputs(f, HN_UU_X_COMMENT_TO, p->text);
		hd = del_header(HD_F_TO, hd);
	}

	return hd;
}

/*
 * Gibt einen String inhalt aus, der gemaess RFC822 umbrochen wird.
 * Ein '\t' im auszugebenden String erzwingt einen Umbruch (es sei denn,
 * es ist innerhab eines "" quoted-string).
 * *col gibt eine Spalte an, in der wir beginnen und steht nach der
 * Rueckkehr auf der derzeitigen Position.
 *
 * Gibt =kein= Zeilenende-Zeichen aus! (Im Gegensatz zu foldputs(), s.u.)
 */
#define MAXCOL 79
void foldputs_no_eol(FILE *f, char *inhalt, int *col)
{
	int i, in_quoted_string, in_quoted_string2;
	char *p, *p1, *lastspace;
	char ch;

	p=inhalt;
	while (*p) {
		lastspace=NULL;
		in_quoted_string=0;
		/* move MAXCOL characters forward; if not in quoted-string,
		 * stop at a tab character and rememer position of final space 
		 */
		for(p1=p; *p1 && (*col)<MAXCOL; p1++, (*col)++) {
			if (*p1 == '\"')
				in_quoted_string = !in_quoted_string;
			if (! in_quoted_string) {
				if (*p1 == '\t')
					break;
				if (*p1 == ' ')
					lastspace=p1;
			}
		}
		if ((*col) == MAXCOL && *p1 != '\t') {
			/* There was no tab char. See if there will be a tab
			 * or the end of the string
			 * within the next 7 characters, then we wrap there.
			 */
			in_quoted_string2 = in_quoted_string;
			for (i=0; 
				i<7 && p1[i] && 
				 (p1[i] != '\t' || in_quoted_string2) ; 
				i++)
				if (p1[i] == '\"')
					in_quoted_string2 = !in_quoted_string2;
			if (i<7)
				p1+=i;
			else {
				/* Move to last space, if any */
				if (lastspace)
					p1=lastspace;
				else { /* else, look for next one */
					while(*p1 && (!isspace(*p1) || 
							in_quoted_string)) {
						if (*p1 == '\"')
							in_quoted_string =
							    !in_quoted_string;
						p1++;
					}
				}
			}
		}
		ch = *p1;
		*p1='\0';
		fputs(p, f);
		*p1=ch;
		while(*p1 && isspace(*p1))
			p1++;
		if (*p1) {
			fputs(eol, f);
			putc('\t', f);
			(*col)=10;
		}
		p=p1;
	}
}

/* Gibt einen Header RFC822-umbrochen aus. Wenn hd != NULL, wird
 * der Name des Headers, gefolgt von einem ": ", vorangestellt.
 * ansonsten wird davon ausgegangen, dass es sich um eine neue Zeile
 * handelt und ein '\t' vorangestellt.
 *
 * Gibt am Ende ein eol aus.
 */
void foldputs(FILE *f, char *hd, char *inhalt) {
	int col;

	if(NULL != hd)
	{
		fputs(hd, f); fputs(": ", f);
		col = strlen(hd)+2;
	} else {
		fputs("\t", f);
		col = 10;
	}
	foldputs_no_eol(f, inhalt, &col);
	fputs(eol, f);
}

/*
 * Gibt einen Header aus, der aus mehreren gleichartigen 
 * ZC-Headern zusammengesetzt ist. hd wird wie in foldputs
 * verwendet.
 */
void foldputh(FILE *f, char *hd, header_p t)
{
	int col;
	header_p p;

	if(NULL != hd)
	{
		fputs(hd, f); fputs(": ", f);
		col = strlen(hd)+2;
	} else {
		fputs("\t", f);
		col = 10;
	}
	for (p=t; p; p=p->other)
	{
		foldputs_no_eol(f, p->text, &col);
		if (NULL!= p->other) {
			fputs(", ", f);
			col += 2;
		}
	}	
	fputs(eol, f);
}

/*
 * Gibt einen Header aus, der aus Mailadressen besteht. Es werden
 * diejenigen ausgegeben, die ein '@' enthalten, sie werden dafr
 * durch mime_encode "gejagt".
 */
void foldputaddrs(FILE *f, char *hd, header_p t)
{
	int col;
	header_p p;
	char *mime_name, *rna;

	if(NULL != hd)
	{
		fputs(hd, f); fputs(": ", f);
		col = strlen(hd)+2;
	} else {
		fputs("\t", f);
		col = 10;
	}
	for (p=t; p; p=p->other) 
	{
		if(strchr(p->text, '@')) {
			if (col > 40) {
				fputs(eol, f); fputs("\t", f);
				col = 10;
			}
			if ((rna = index(p->text, '(')) != NULL)
				pc2iso(rna, strlen(rna));
			mime_name=mime_address(p->text);
			fprintf(f, "%s%s", mime_name, p->other ? ", " : "");
			col += strlen(mime_name) + (p->other ? 2:0);
			dfree(mime_name);
		}
	}	
	fputs(eol, f);
}

void u_f_and_all(FILE *f, header_p hd) {
	header_p p, p1;
	char *cp, *praefix;
	int offset, do_encode, hd_len;

        /* U-, F- und ZConnect-eigene unbekannte
         * Headerzeilen richtig umwandeln:
         */ 
        for (p=hd; p; p=p->next) {
		/* finde den Typ und setze praefix, do_encode und offset */
		cp = p->header;
		offset=0;
		praefix=NULL;
		do_encode=0;
		if (*cp && cp[1]=='-') {
			if (*cp == 'U' || *cp == 'u') {
				/* In ZC durchgereichte RFC-Hd-Zeile */
				offset=2;
			}
			else 
			if (*cp == 'F' || *cp == 'f') {
				/* In ZC durchgereichte Fido-Hd-Zeile */
				offset=2;
				praefix="X-FTN-";
			}
		}
		else {
			/* ZC-Headerzeile, von uns nicht behandelt */
			praefix="X-ZC-";
			do_encode=1;
		}
		hd_len = strlen(cp) - offset;

                for (p1=p; p1; p1=p1->other) {
			if (praefix)
				fputs(praefix, f);
			pc2iso(p1->text, strlen(p1->text));
			if (do_encode) {
				cp = mime_encode(p1->text, hd_len);
				foldputs(f, p1->header+offset, cp);
				dfree(cp);
			}
			else
				foldputs(f, p1->header+offset, p1->text);
                }
	}
}
