#!/bin/bash
#
#	$Id: Backinout.sh,v 1.3 1998/03/04 02:04:14 moritz Exp $
#		(C) 1996,97 DFG/M.Watermann, Hannover, FRG
#
#		Script zum Sichern der Brettpuffer-Backups
#
#-----------------------------------------------------------------------------
set -h -p

# Name der Datei mit weiteren Konfigurations-Angaben:
KonfigDat="/home/helpers/mbox/zconnect/gate.vars"

#-----------------------------------------------------------------------------
#	(you shouldn't change anything beyond this point!)
#
#	$Log: Backinout.sh,v $
#	Revision 1.3  1998/03/04 02:04:14  moritz
#	neue Version
#	
#	Revision 1.7  1997/11/22 20:00:20  matthias
#	Konfigurations-Datei konfigurierbar gemacht
#
#	Revision 1.6  1996/09/11 19:03:20  matthias
#	obsolete Zeilen entfernt;
#
#	Revision 1.5  1996/09/06 12:24:23  matthias
#	Crontab-Beispiel am Ende angehaengt;
#	globale Variablen R/O gemacht;
#	Aufruf-Hinweis ergaenzt;
#
#	Revision 1.4  1996/08/06 17:56:41  matthias
#	generelle "umask 007" fuer Datei-Erzeugung;
#	sollte jetzt als User "mbox.news" laufen;
#
#	Revision 1.3  1996/08/04 12:16:07  matthias
#	'chown' bei erzeugten Archiven ergaenzt;
#
#	Revision 1.2  1996/08/03 17:55:08  matthias
#	verbose-Flag aus tar-Aufruf entfernt;
#	'ZipIt' loescht nicht mehr selbst die archivierten Dateien; 
#
#	Revision 1.1  1996/08/01 22:24:31  matthias
#	Initial revision; Aufnahme ins RCS
#
#-----------------------------------------------------------------------------
#	interne Arbeitsvariablen:
: ${1:?"Aufruf: ${0##*/} -t | -m | -j"}
KonfigDat="${KonfigDat:-/home/helpers/mbox/zconnect/gate.vars}"
[ -s ${KonfigDat} ] || exit 1
. ${KonfigDat}					# Gateway-Variablen einlesen
declare -r BackInTage=${BackInDir}/Tage
declare -r BackOutTage=${BackOutDir}/Tage
declare -r BackInMonate=${BackInTage}/Monate
declare -r BackOutMonate=${BackOutTage}/Monate
declare -r BackInJahre=${BackInMonate}/Jahre
declare -r BackOutJahre=${BackOutMonate}/Jahre
declare -r TagesDatum=`date +%Y%m%d`
declare -r MonatsDatum=`date +%Y%m`
declare -r JahresDatum=`date +%Y`
declare -r ArcExt=".tar.gz"

#-----------------------------------------------------------------------------
#
function ChkDir() {
#	erzeugt Verzeichnis $1 ggf. neu
#
local Dir=${1}
  [ $# -lt 1 ] && return 1
  [ -d ${Dir} ] && return 0
  mkdir -p ${Dir}
} # ChkDir

#-----------------------------------------------------------------------------
#
function ZipIt() {
#	Aufruf: ZipIt ArchivName File(s)
#
local Ziel=${1}
  shift ; [ $# -lt 1 ] && return 1
  tar czf ${Ziel}${ArcExt} $* &>/dev/null
} # ZipIt

#-----------------------------------------------------------------------------
#
function ZipRange() {
#	Aufruf: BackupDir DatumsMaske Dateiendung
#
local BackupDir=${1}
local DatumsMaske=${2}
local DateiExt=${3}
  [ $# -lt 2 ] && return
  [ -d ${BackupDir} ] || return 1
  builtin cd ${BackupDir} || return 1
  builtin cd ..
  ZipIt ${BackupDir}/${DatumsMaske} ${DatumsMaske}${DateiExt} || return 1
  rm -f ${DatumsMaske}${DateiExt}
  return 0
} # ZipRange

#-----------------------------------------------------------------------------
#
function ZipTag() {
#	macht Tagessicherung
#
  ChkDir ${BackInTage} && ZipRange ${BackInTage} ${TagesDatum} "*.brt"
  ChkDir ${BackOutTage} && ZipRange ${BackOutTage} ${TagesDatum} "*.brt"
} # ZipTag

#-----------------------------------------------------------------------------
#
function ZipMonat() {
#	macht Monatssicherung
#
  [ -f ${BackInTage}/${TagesDatum}${ArcExt} ] || ZipTag
  ChkDir ${BackInMonate} && ZipRange ${BackInMonate} ${MonatsDatum} "*${ArcExt}"
  ChkDir ${BackOutMonate} && ZipRange ${BackOutMonate} ${MonatsDatum} "*${ArcExt}"
} # ZipMonat

#-----------------------------------------------------------------------------
#
function ZipJahr() {
#	macht Jahressicherung
#
  [ -f ${BackInMonate}/${MonatsDatum}${ArcExt} ] || ZipMonat
  ChkDir ${BackInJahre} && ZipRange ${BackInJahre} ${JahresDatum} "*${ArcExt}"
  ChkDir ${BackOutJahre} && ZipRange ${BackOutJahre} ${JahresDatum} "*${ArcExt}"
} # ZipJahr

#-----------------------------------------------------------------------------
#
# Main()
#
umask 007
ChkDir ${BackInDir}
ChkDir ${BackOut}

case "${1}" in
  "-t"|"-T"|t|T)
	ZipTag						# Tagessicherung
	;;
  "-m"|"-M"|m|M)
	ZipMonat					# Monatssicherung
	;;
  "-j"|"-J"|j|J)
	ZipJahr						# Jahressicherung
	;;
  *)	echo -e "\n\tAufruf: ${0##*/} T | M | J\n"
	exit 1 ;;
esac

exit 0

#-----------------------------------------------------------------------------
#
# Das Script ist hier in '/etc/crontab' so eingebunden:
#
# ZC-Gateway Daten archivieren:
53 23 1-30 1,3,5,7,8,10,12 * mbox /home/helpers/mbox/zconnect/Backinout.sh T
53 23  31    1,3,5,7,8,10  * mbox /home/helpers/mbox/zconnect/Backinout.sh M
53 23 1-29     4,6,9,11    * mbox /home/helpers/mbox/zconnect/Backinout.sh T
53 23  30      4,6,9,11    * mbox /home/helpers/mbox/zconnect/Backinout.sh M
53 23 1-27        2        * mbox /home/helpers/mbox/zconnect/Backinout.sh T
53 23  28         2        * mbox /home/helpers/mbox/zconnect/Backinout.sh M
53 23  31        12        * mbox /home/helpers/mbox/zconnect/Backinout.sh J
#
# _EoF_
