#! /bin/sh
# $Id: prearc.SH,v 1.10 2000/06/26 13:52:49 moritz Exp $
#
#   PreARC: Komprimieren der ZCONNECT/JANUS Netcalls
#	    Konvertieren der ZNETZ-Netcalls nach Netcall-3.8 und komprimieren.
#           Wird aus der Crontab regelmaessig (z.B. einmal pro Stunde)
#	    aufgerufen.
#
# 	    Einmal am Tag werden zu alte Daten aus Netcallpuffern
#           geloescht. Die Konfiguration dafuer steckt in den einzelnen
#           System-Konfigurations-Dateien als 'Keep-Brt:' und 'Keep-Prv:';
#	    wenn sie dort fehlen, werden Standardwerte aus der Konfigurations-
#           Datei 'config' genommen (mit denselben Header-Namen).
#
#   Benutzt: einpack, auspack
#
#   Privileges: laeuft bei mir derzeit als root (daher auch chown...)
#

LOCK=@SPOOLDIR@/locks/prearc.lck
CONFIG=@DESTDIR@/config
SHRINK_TS_NAME=shrink.timestamp
SHRINK_TS_DIR=@SPOOLDIR@
SHRINK_TS="$SHRINK_TS_DIR/$SHRINK_TS_NAME"

# Das shlock vom INN macht viel bessere Locks. Es erkennt
# beispielsweise auch liegengebliebene Lockfiles. Dann bleiben
# nicht mehr drei Tage lang alle Nachrichten liegen, weil
# irgendjemand den Gate-Rechner gebootet hat...

if [ -x /usr/bin/shlock ]; then
	/usr/bin/shlock -p $$ -f ${LOCK} ||{ echo "$0: Es laeuft bereits ein anderer Prozess" >&2; ls -l $LOCK >&2; exit 1; }
	trap 'rm -f ${LOCK} ; exit 1' 1 2 3 15
	trap 'rm -f ${LOCK} ; exit 0' 0
else
	trap "rm -f $LOCK.$$; exit 0" 0 1 2 3
	
	echo $$ > $LOCK.$$
	if (umask 222; ln $LOCK.$$ $LOCK) 2>/dev/null
	then
		# OK -> Datei gab es noch nicht
		trap "rm -f $LOCK $LOCK.$$; exit 0" 0 1 2 3 
		rm -f $LOCK.$$
	else
		# Fehler -> Lock existierte
		echo "$0: es laeuft bereits ein anderer Prozess" >&2
		ls -l $LOCK >&2
		exit 0
	fi
fi

SPOOL=@SPOOLDIR@

PATH=@DESTDIR@:@PATHADD@:$PATH; export PATH

# zbatchsmtp

# shrink
# find out if shrinking is now (once a day)
DO_SHRINK=no
if test ! -f $SHRINK_TS; then
	DO_SHRINK=yes
	touch $SHRINK_TS || DO_SHRINK=no
fi
test `find $SHRINK_TS -mtime +0 -print|wc -l` -eq 1 \
	&& touch $SHRINK_TS && DO_SHRINK=yes
if [ $DO_SHRINK = yes ]; then
	KEEP_PRV_GLOBAL=`grep -i '^Keep-Prv:' $CONFIG | awk '{print $2}'`
	KEEP_BRT_GLOBAL=`grep -i '^Keep-Brt:' $CONFIG | awk '{print $2}'`
	test -z "$KEEP_PRV_GLOBAL" && KEEP_PRV_GLOBAL=0
	test -z "$KEEP_BRT_GLOBAL" && KEEP_BRT_GLOBAL=0
fi

cd @DESTDIR@/systems || exit 1

for sys in *
do
  if [ -d $SPOOL/$sys ]
  then
	arc=`fgrep -i 'ArcerOut:' @DESTDIR@/systems/$sys | awk '{print $2}'`
	fmt=`fgrep -i 'X-Call:' @DESTDIR@/systems/$sys | awk '{print $2}'`
	if [ $DO_SHRINK = yes ]; then
		KEEP_PRV=`fgrep -i 'Keep-Prv:' @DESTDIR@/systems/$sys \
					|awk '{print $2}'`
		KEEP_BRT=`fgrep -i 'Keep-Brt:' @DESTDIR@/systems/$sys \
					|awk '{print $2}'`
		test -n "$KEEP_PRV" || KEEP_PRV=$KEEP_PRV_GLOBAL
		test -n "$KEEP_BRT" || KEEP_BRT=$KEEP_BRT_GLOBAL
	fi

	SYSLOCK=$SPOOL/$sys/prearc.lck		# Prearc laueft
	NETLOCK=$SPOOL/$sys/netcall.lck		# Netcall laeuft

	if cd $SPOOL/$sys ; then :; else
		echo "$0: cd $SPOOL/$sys failed"
		continue
	fi

	# Lock fuer dieses System erzeugen
	# (das ist so einfach, weil niemand ausser 'prearc' dort Locks erzeugt,
	# und ein anderes 'prearc' ist bereits ausgeschlossen)
	echo $$ > $SYSLOCK

	if [ $DO_SHRINK = yes ]; then
		DELFB=""
		DELFP=""
		if [ $KEEP_BRT -gt 0 -o $KEEP_PRV -gt 0 ]; then
			fname=`auspack $arc ../$sys filename`
			if [ $? = 0 ]; then
				rm -f $fname
				test $KEEP_BRT -gt 0 && DELFB=`find . -mtime \
					+${KEEP_BRT} -name \*.brt -print`
				test $KEEP_PRV -gt 0 && DELFP=`find . -mtime \
					+${KEEP_PRV} -name \*.prv -print`
				test -n "$DELFB" && rm -f $DELFB
				test -n "$DELFP" && rm -f $DELFP
			fi
		fi
	fi

	# System nur bearbeiten, wenn (a) kein Netcall laeuft und (b) Arbeit
	# vorhanden ist
	WRK=`ls *.prv *.brt 2>/dev/null`
	if [ \( ! -f $NETLOCK \) -a \(  "$WRK" != "" \) ]; then
		if [ $fmt = "JANUS" ]
		then
			if einpack $arc ../$sys $WRK; then
				rm -f $WRK
			else
				echo "$0: einpack $arc ../$sys $WRK failed" >&2
			fi
			chown mbox ../$sys.* 2>/dev/null
		fi
		if [ $fmt = "ZCONNECT" ]
		then
			einpack $arc ../prv.$sys *.prv *.eil *.neu 2>/dev/null
			einpack $arc ../brt.$sys *.brt 2>/dev/null
			chown mbox ../prv.$sys.* 2>/dev/null
			chown mbox ../brt.$sys.* 2>/dev/null
		fi
	fi

	# Ok, dieses System ist fertig - Lock wieder freigeben
	rm -f $SYSLOCK
  fi
done

